clear all
close all

%tank parameters
Ts=0.1;
A_tank=0.1;
Kp=0.002;
R=0.000001;
Q=[0.01,0;0,0.0001];
G=[1,0;0,1];
I=[1,0;0,1];

%setting parameters
L= 10000;
Pp = [0.25,0;0,1];
t= 0:0.1:999.9;
u= 3*ones(L,1);
e=ones(L,1);
x1=zeros(L,1);
x2=zeros(L,1);
x1p=zeros(L,1);
x2p=zeros(L,1);
x1c=zeros(L,1);
x2c=zeros(L,1);
K=zeros(2,1);
v=0.000001+0.000001.*randn(L,1);
w = 0.000001+0.000001.*randn(L,2);
ik=1;
ek=1;

%State space model
A_con=[0,-1/A_tank;0,0];
B_con=[Kp/A_tank;0];
C_con=[1,0];
D_con=[0];
sys_con=ss(A_con,B_con,C_con,D_con);

%discretization 
sys_dis=c2d(sys_con, Ts);
A= [1,-(Ts/A_tank);0,1];
B=sys_dis.b;
C=sys_dis.c;
D=sys_dis.d;

%kalman gain
%[Ks,Pp,Pc,E]=dlqe(A,G,C,Q,R);

%Kalman gain manually
%cycle while is run until the error will be less than 0.0005
while abs(e)>0.00005
    x1(1)=0.5;
    %kalman gain 
    K= Pp*C'*inv(C*Pp*C'+R);
    Pc = (I-K*C)*Pp;
    Pp=A*Pc*A'+G*Q*G';
    %determine of error
    y(ik)=x1(ik)+v(ik,1);
    yp(ik)= x1p(ik);
    e=y(ik)-yp(ik)
    
    %state correction 
    x1c(ik)=x1p(ik)+K(1)*e;
    x2c(ik)=x2p(ik)+K(2)*e;
    
    %state estimaton
    x1p(ik+1)=x1c(ik)+(Ts/A_tank)*(Kp*u(ik)-x2c(ik));
    x2p(ik+1)=x2c(ik);
    %step +1
    ik=ik+1;
end
%Kalman filter
%setting parameters
e=1;
x1=zeros(L,1);
x2=0.0006*ones(L,1);
x1p=zeros(L,1);
x2p=zeros(L,1);
x1c=zeros(L,1);
x2c=zeros(L,1); 
y=zeros(L,1);
yp=zeros(L,1);

for i=1:10000
    x1(1)=0.5;
    x2(1)=0.006;
    x1p(1)=x1(1);
    x2p(1)=0;
    %determine of error
    y(i)=x1(i);
    yp(i)= x1p(i);
    e(i)=y(i)-yp(i);
    
    %state correction 
    x1c(i)=x1p(i)+K(1)*e(i);
    x2c(i)=x2p(i)+K(2)*e(i);
    
    %state prediction estimaton
    x1p(i+1)=x1c(i)+(Ts/A_tank)*(Kp*u(i)-x2c(i));
    x2p(i+1)=x2c(i);
    
    %state space model 
    x1(i+1)= x1(i)+(Ts/A_tank)*(Kp*u(i)-x2(i))+w(i,1);
    x2(i+1)= x2(i)+w(i,2); 
end;

%plot
figure(1)
plot(t,y,'b-',t,yp,'r-.');
title('Level in tank y vs yp')
xlabel('Time (s)');
ylabel('Measurement');
legend('Estimated Value','Real Value')
figure(2)
plot(t,x1,'b-',t,x1c,'r-.');
title('Level d\_est = red. d\_sim = blue.')
xlabel('Time (s)');
ylabel('Measurement');
legend('Estimated Value','Real Value')
